 # titris.tcl---PRISMA/CLARA TITRIS Control module control frame



set frame "TITRIS"
set frametitle "TITRIS Control module"
set icontitle "TITRIS"

set modulename titris
set module 1

#    assume the Titris module is in the first VXI crate in the VME server list - this will be the lowest numbered VXI crate
set crates [inform vxicontrol inquire VXICrateList]
set titris_clara [inform vxicontrol VXIcrate2server [first $crates]]

set titris_prisma prisma

set clara_inuse 1
set prisma_inuse 0

loadsource common.vme

proc setup {} {
    global titris_clara
    inform vmecontrol Add_TITRIS_Module titris#1 $titris_clara 0:0:0x08000000:0:0
}

#proc write-vme-register {r v} {
#    global CurrentVMEServer
#    insert-log "write-vme-register $CurrentVMEServer $r $v"
#}
#proc read-vme-register {r} {
#    global CurrentVMEServer
#    insert-log "read-vme-register $CurrentVMEServer $r"
#    return 0x5555
#}

proc select_module {W} {
    global CurrentVMEServer modulename module
    global titris_clara titris_prisma
    
    if {$W == "C"} {set module 1; set CurrentVMEServer $titris_clara}
    if {$W == "P"} {set module 2; set CurrentVMEServer $titris_prisma}

#   set CurrentVMEServer [VMEmodule2server $modulename#$module]
}


proc set-register {register v} {
    global modulename module
    write-vme-register $modulename#$module.$register $v
#    click-redisplay
}

proc click-master {args} {set-register Control 4}
proc click-slave  {args} {set-register Control 0x20} 
proc click-go     {args} {set-register Control 4; set-register Control 2}
proc click-halt   {args} {set-register Control 4; set-register Control 0x20}

proc click-calibrate {args} {
    global modulename module

    set success 0
    set-footer "Calibrating cable delay - please wait"

    write-vme-register $modulename#$module.Control 0x20

    write-vme-register $modulename#$module.Control 0x100

    for {set i 0} {$i < 5} {incr i} {
        after 1000
        set r [expr [read-vme-register $modulename#$module.Control] & 0xffff]
        if {$r == [expr 0x8900]} {
            set success 1 
            break
        }
    }

    :modify PDELR :value [read-vme-register $modulename#$module.CableDelay]

    if {$success} {
       set-footer "OK: cable delay calibrated"
    } else {
       set-footer "ERROR: calibrating cable delay"
    }
    return $success
}


proc click-software {} {
   global modulename
   inform [set modulename]sftwre start
}

proc click-redisplay {} {
    global clara_inuse prisma_inuse

    if {$clara_inuse != 0} {TITRIS_display C}
    if {$prisma_inuse != 0} {TITRIS_display P}
}

proc calibrate_all {} {

    set r [midas-confirmation "You are about to recalibrate the TITRIS system\n\
                          Are you sure you wish to continue"]
    if {$r == 0} then {return}

    log "[date]: Calibrating TITRIS system"
    titris_calibrate
    log "[date]: Finished"
}

proc start args {
   global started

   if {$started == 0} then {
       make-frame
       EGbindings
       
       setup
   }
   set started 1
   open-frame

   clear-footer
   clear-busy
   set EGreport 0

   select_module 1
   click-redisplay
}

proc click-quit {} {close-frame}

proc finish args {destroy-frame}

proc click arg {
   global debug actionlist
   global directory program clara_inuse prisma_inuse
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {
      ENGINEERING       {click-$v}
      CMASTER           {select_module C; click-master}
      PMASTER           {select_module P; click-master}
      CSLAVE            {select_module C; click-slave}
      PSLAVE            {select_module P; click-slave}
      CGO               {select_module C; click-go}
      PGO               {select_module P; click-go}
      CHALT             {select_module C; click-halt}
      PHALT             {select_module P; click-halt}
      CCALIBRATE        {select_module C; click-calibrate}
      PCALIBRATE        {select_module P; click-calibrate}
      CINUSE            {set clara_inuse $v}
      PINUSE            {set prisma_inuse $v}
      default {
        set p "click-[lowercase $w]"
        if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
   click-redisplay
}


proc make-frame {} {
  global env
  global clara_inuse prisma_inuse

set rows 8
set cols 80

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" 

   :panel CLARA  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :message CM1 :label "CLARA" :bold T

     :checkbox CINUSE :label "" :value $clara_inuse
     :text CSTATUS   :label "Status register"  :w 6 :deaf T
     :text CCONTROL  :label "Control register" :w 6 :deaf T
     :text CDELR     :label "Cable Delay"  :deaf T  :w 6
     :next-row T

     :message CM2 :label "Data Register" :bold  T
     :text CDATAHW  :label "High Word"    :w 6 :deaf T
     :text CDATAMW  :label "Middle Word"  :w 6 :deaf T
     :text CDATALW  :label "Low Word"     :w 6 :deaf T

     :next-row T
     :button CMASTER    :label "Set module to master mode"
     :button CSLAVE     :label "Set module to slave mode"
     :button CCALIBRATE :label "Calibrate cable delay"

     :next-row T
     :button CGO    :label "GO"  :w 8
     :button CHALT  :label "HALT"  :w 8
     :checkbox CCGO  :label "GO set" :deaf T
     :checkbox CCMTM :label "Master set" :deaf T
     :checkbox CCRES :label "Reset set" :deaf T
     
   :panel PRISMA  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :message PM1 :label "PRISMA" :bold T

     :checkbox PINUSE :label "" :value $prisma_inuse
     :text PSTATUS   :label "Status register"  :w 6 :deaf T
     :text PCONTROL  :label "Control register" :w 6 :deaf T
     :text PDELR     :label "Cable Delay"  :deaf T  :w 6
     :next-row T

     :message PM2 :label "Data Register" :bold  T
     :text PDATAHW  :label "High Word"    :w 6 :deaf T
     :text PDATAMW  :label "Middle Word"  :w 6 :deaf T
     :text PDATALW  :label "Low Word"     :w 6 :deaf T

     :next-row T
     :button PMASTER    :label "Set module to master mode"
     :button PSLAVE     :label "Set module to slave mode"
     :button PCALIBRATE :label "Calibrate cable delay"

     :next-row T
     :button PGO    :label "GO"  :w 8
     :button PHALT  :label "HALT"  :w 8
     :checkbox PCGO  :label "GO set" :deaf T
     :checkbox PCMTM :label "Master set" :deaf T
     :checkbox PCRES :label "Reset set" :deaf T

  :panel P2 :layout V
    :galley LIST :rows $rows :cols $cols :deaf T  :font variable

   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 10
      :vskip 10
      :button REDISPLAY  :command click-redisplay
      :menubutton ENGINEERING :label "Engineering"  :menu [list software]
      :button DOALL :label "Calibrate TITRIS system" :command calibrate_all

  open-frame

  :galley_size LIST $rows $cols

}

proc click-configure {W} {:galley_resize LIST}

proc log {s} {:modify LIST :strings $s\n}

proc titris_calibrate {} {
    global clara_inuse prisma_inuse

#    first set all TITRIS modules to slave

    if {$clara_inuse != 0} {
        log "[date]: Setting CLARA TITRIS to slave mode"
        select_module C
        set result [click-slave]
        log "[date]: $result"
    }

    if {$prisma_inuse != 0} {
        log "[date]: Setting PRISMA TITRIS to slave mode"
        select_module P
        set result [click-slave]
        log "[date]: $result"
    }

#    first master TITRIS module to master

    if {$prisma_inuse != 0} {
        log "[date]: Setting CLARA TITRIS to master mode"
        select_module C
        set result [click-master]
        log "[date]: $result"
    }

#    now calibrate all slaves 

     if {$prisma_inuse != 0} {
         log "[date]: Calibrating PRISMA TITRIS"
         select_module P
         set result [click-calibrate]
         log "[date]: $result"
     }

#    set master TITRIS module to GO

    if {$clara_inuse != 0} {
        log "[date]: Setting CLARA TITRIS to GO"
        select_module C
        set result [click-go]
        log "[date]: $result"
    }

}

proc TITRIS_display {W} {
    global module modulename
    
    select_module $W

    set register Status
    set vl [read-vme-register $modulename#$module.$register]
    :modify [set W]STATUS :value  [format "0x%04x" [expr $vl & 0xffff]]

    set register Control
    set vl [read-vme-register $modulename#$module.$register]
    :modify [set W]CONTROL :value  [format "0x%04x" [expr $vl & 0xffff]]

    set register Control.GO
    set vl [read-vme-register $modulename#$module.$register]
    :modify [set W]CGO :value $vl

    set register Control.MASTER
    set vl [read-vme-register $modulename#$module.$register]
    :modify [set W]CMTM :value $vl

    set register Control.RESET
    set vl [read-vme-register $modulename#$module.$register]
    :modify [set W]CRES :value $vl

    set register CableDelay
    set vl [read-vme-register $modulename#$module.$register]
    :modify [set W]DELR :value $vl

    set register Data.LW
    set vl [read-vme-register $modulename#$module.$register]
    :modify [set W]DATALW :value  [format "0x%04x" [expr $vl & 0xffff]]

    set register Data.MW
    set vl [read-vme-register $modulename#$module.$register]
    :modify [set W]DATAMW :value  [format "0x%04x" [expr $vl & 0xffff]]

    set register Data.HW
    set vl [read-vme-register $modulename#$module.$register]
    :modify [set W]DATAHW :value  [format "0x%04x" [expr $vl & 0xffff]]
}


