

set frame XDAQ
set frametitle "XDAQ Client"
set icontitle "XDAQ"

set ConfigurationFile prisma_cl.xml

set HostIDs {}
set HostAddresses {}
set HostPorts {}

set HostAddress 0
set HostPort 0
set Target 0

set Applications {}
set TargetAddresses {}

set EnableSequence {}
set DisableSequence {}

set HostID 0   
set TargetAddress 0

set LastMethodName ""

set HostActions [list "Configure ALL Hosts" "Configure Current Host" \
                      "Query Current Host State"  \
                      "Halt Current Host" "Reset Current Host" \
		      "Query Current Host"]
set ApplActions [list "Configure ALL Applications" "Configure Current Application" \
                      "Query Current Application State" "Display Current Application" \
                      "Enable ALL Applications" "Disable ALL Applications" \
		      "Enable Current Application" "Disable Current Application" \
                      "Halt Current Application" "Reset Current Application" "Query Current Application"]

proc make-frame {} {
   global env
   global HostIDs TargetAddresses
   global HostActions ApplActions

set rows 15
set cols 80

  :frame [self frame-name] :label "[self frame-title]" \
     :icon-name "[self icon-title]"

  :panel P1 :layout V :padx 5 :pady 5
    :layout H; :col-gap 20
     :text CONFIGURATIONFILE :label "XDAQ configuration file" :deaf T
     :button RELOAD  :label "reload"  :command "click_reload; redisplay"
     :next-row T
     :galley CONFIGURATION :rows 8 :cols $cols :font variable
     
  :panel P2 :layout V :padx 5 :pady 5 
    :layout H; :col-gap 5; :row-gap 5
    :choice-stack HOSTS :label "Current Host" :strings $HostIDs :value 0 :valuex 16c
    :hskip 15
    :choice-stack HOSTACTIONS :label "Host Actions" :strings $HostActions :valuex 20c
   :next-row T
    :choice-stack APPLS :label "Current Application" :strings [lindex $TargetAddresses 0] :value 0 :valuex 16c
    :hskip 8
    :choice-stack APPLACTIONS :label "Application Actions" :strings $ApplActions :valuex 20c

  :panel P3 :layout V :padx 5 :pady 5
     :galley STATUS :rows 10 :cols $cols :font variable

  :panel P4 :layout V :padx 5 :pady 5
    :galley LIST :rows $rows :cols $cols :font variable
    
  :panel P5 :layout H :padx 5 :pady 5
    :layout H; :col-gap 20; :row-gap 5
    :button D1 :label "Show last request" :command click_request
    :button D2 :label "Show last reply" :command click_reply
    :button D3 :label "Clear" :command ":modify LIST :empty T"

   open-frame

#  :galley_size LIST $rows $cols

}

proc click_reload {args} {
    global ConfigurationFile
    global HostIDs HostAddresses HostPorts 
    global TargetAddresses Applications
    global EnableSequence DisableSequence
    global BUQuery RMQuery RUIQuery
   upvar #0 browserdirectory directory

    cd $directory

    source xdaq.defn.tcl
    
    source XDAQClient.tcl
}

proc redisplay {} {
    global ConfigurationFile
    global HostIDs HostAddresses HostPorts 
    global TargetAddresses Applications
    global EnableSequence DisableSequence
    global HostID TargetAddress
    
    set list [lindex $TargetAddresses [lsearch $HostIDs $HostID]]
    if {[lsearch $list $TargetAddress] < 0} {set TargetAddress [lindex $list 0]}
    :modify APPLS :strings $list :value [lsearch $list $TargetAddress]
    
    :modify CONFIGURATIONFILE :value $ConfigurationFile
    
    :modify CONFIGURATION :empty T
    
    :modify CONFIGURATION :strings "Configuration file = $ConfigurationFile\n"
    
    set i 0
    foreach ID $HostIDs {
        set HostAddress [lindex $HostAddresses $i]
        set HostPort [lindex $HostPorts $i]
        
        set TargetAddressList [lindex $TargetAddresses $i]
        set ApplicationList [lindex $Applications $i]

        :modify CONFIGURATION :strings "\tHost id = $ID; URL = //http/$HostAddress:$HostPort\n"
        
        set j 0
        foreach TargetAddr $TargetAddressList {
            set Application [lindex $ApplicationList $j]
            
            :modify CONFIGURATION :strings "\t\tApplication = $Application; TargetAddress = $TargetAddr\n"
            incr j
        }
        incr i
    }
}

proc XDAQ_Error {m} {
    global LastMethodName
    global HostAddress HostPort
    global Target
    
    insert $m
    
    midas-warning "\
          Error in XDAQ request\n\
	  Host address $HostAddress: Host Port $HostPort\n\
          Target $Target: Method $LastMethodName\n\
	  Response: $m"
}

proc configureallhosts {args} {
    global HostIDs HostAddresses HostPorts 
    global ConfigurationFile
    global LastMethodName
    global HostAddress HostPort Target
   upvar #0 browserdirectory directory
   
     cd $directory

#    to configure all XDAQ executives

    set i 0
    foreach ID $HostIDs {
      set HostAddress [lindex $HostAddresses $i]
      set HostPort [lindex $HostPorts $i]
  
      SOAP::configure XDAQ_ExecConfigure -proxy http://$HostAddress:$HostPort/
      set LastMethodName XDAQ_ExecConfigure
      set Target $ID
      set z [catch {XDAQ_ExecConfigure $ID $ConfigurationFile} m]
      if {$z != 0} then {XDAQ_Error "$m"} else {insert "$LastMethodName=> $m"}
  
      incr i
    }
}

proc resetallhosts {args} {
    global HostIDs HostAddresses HostPorts 
    global LastMethodName
    global HostAddress HostPort Target

#    to RESET all XDAQ executives

    set i 0
    foreach ID $HostIDs {
      set HostAddress [lindex $HostAddresses $i]
      set HostPort [lindex $HostPorts $i]
  
      SOAP::configure XDAQ_ExecReset -proxy http://$HostAddress:$HostPort/
      set LastMethodName XDAQ_ExecReset
      set Target $ID
      set z [catch {XDAQ_ExecReset $ID} m]
      incr i
    }
}

proc Condconfigureallhosts {args} {
    global HostIDs HostAddresses HostPorts 
    global ConfigurationFile
    global LastMethodName
    global HostAddress HostPort Target
   upvar #0 browserdirectory directory
   
     cd $directory

#    to configure all XDAQ executives which are in the Halted state

    set i 0
    foreach ID $HostIDs {
      set HostAddress [lindex $HostAddresses $i]
      set HostPort [lindex $HostPorts $i]
      
      SOAP::configure XDAQ_ReadParameter -proxy http://$HostAddress:$HostPort/
      SOAP::configure XDAQ_ReadParameter -params [list stateName string]
      set LastMethodName "XDAQ_ReadParameter stateName"
      set Target 0
      set z [catch {XDAQ_ReadParameter -attr "targetAddr 0" ""} m]
      if {$z != 0} then {XDAQ_Error "$m"} else {
          insert "$LastMethodName=> $m"
	  if {$m == "Halted"} {
              SOAP::configure XDAQ_ExecConfigure -proxy http://$HostAddress:$HostPort/
              set LastMethodName XDAQ_ExecConfigure
              set Target $ID
              set z [catch {XDAQ_ExecConfigure $ID $ConfigurationFile} m]
              if {$z != 0} then {XDAQ_Error "$m"} else {insert "$LastMethodName=> $m"}
	  }
      }
      incr i
    }
}

proc Condconfigureallapplications {} {
    global HostIDs HostAddresses HostPorts 
    global TargetAddresses
    global LastMethodName
    global HostAddress HostPort Target
    
#    ensure all applications for all XDAQ executives have been configured

    set i 0
    foreach ID $HostIDs {
        set HostAddress [lindex $HostAddresses $i]
        set HostPort [lindex $HostPorts $i]
  
        set TargetAddressList [lindex $TargetAddresses $i]
  
        SOAP::configure XDAQ_Configure -proxy http://$HostAddress:$HostPort/

        foreach TargetAddr $TargetAddressList {
	
	   if {[read_parameter $i $TargetAddr stateName] == "Halted"} then {
               set LastMethodName XDAQ_Configure
	       set Target $TargetAddr
               set z [catch {XDAQ_Configure -attr "targetAddr $TargetAddr"} m]
               if {$z != 0} then {XDAQ_Error "$m"} else {insert "$LastMethodName=> $m"}
	   }
        }
  
        incr i
    }
}

proc configurecurrenthost {args} {
    global HostIDs HostAddresses HostPorts
    global HostID
    global ConfigurationFile
    global LastMethodName
    global HostAddress HostPort Target
   upvar #0 browserdirectory directory

    cd $directory
    
    set i [lsearch $HostIDs $HostID]
    
    set HostAddress [lindex $HostAddresses $i]
    set HostPort [lindex $HostPorts $i]
    
     SOAP::configure XDAQ_ExecConfigure -proxy http://$HostAddress:$HostPort/
     set LastMethodName XDAQ_ExecConfigure
     set Target $HostID
     set z [catch {XDAQ_ExecConfigure $HostID $ConfigurationFile} m]
     if {$z != 0} then {XDAQ_Error "$m"} else {insert "$LastMethodName=> $m"}
}

proc do_currenthost {action} {
    global HostIDs HostAddresses HostPorts
    global HostID
    global ConfigurationFile
    global LastMethodName
    global HostAddress HostPort Target

    set i [lsearch $HostIDs $HostID]
    
    set HostAddress [lindex $HostAddresses $i]
    set HostPort [lindex $HostPorts $i]
    
     SOAP::configure XDAQ_Exec$action -proxy http://$HostAddress:$HostPort/
     set LastMethodName XDAQ_Exec$action
     set Target $HostID
     set z [catch {XDAQ_Exec$action $HostID} m]
     if {$z != 0} then {XDAQ_Error "$m"} else {insert "$LastMethodName=> $m"}
}

proc click_displaycurrentapplication {args} {
    global BUQuery RMquery RUIQuery
    
    set AppName BU
    
    set state [read_parameter 1 15 stateName]
    set data [do_currentapplication Query]
    
    :modify STATUS :empty T
    
    insert_status "BU state is $state\n"
    
    set i 0
    foreach item [set [set AppName]Query] {
        insert_status "$item=[lindex $data $i]"
        incr i
    }
}

proc insert_status {s} {:modify STATUS :strings "$s\t"}

proc do_currentapplication {action} {
    global HostIDs HostAddresses HostPorts
    global HostID TargetAddress
    global LastMethodName
    global HostAddress HostPort Target
    
#    perform action for the current application for the current host
    
    set i [lsearch $HostIDs $HostID]
    
    set HostAddress [lindex $HostAddresses $i]
    set HostPort [lindex $HostPorts $i]
    
    SOAP::configure XDAQ_$action -proxy http://$HostAddress:$HostPort/
    set LastMethodName XDAQ_$action
    set Target $TargetAddress
    set z [catch {XDAQ_$action -attr "targetAddr $TargetAddress"} m]
    if {$z != 0} then {XDAQ_Error "$m"} else {insert "$LastMethodName=> $m"}
    return $m
}

proc do_applications {action} {
    global HostIDs HostAddresses HostPorts 
    global TargetAddresses
    global LastMethodName
    global HostAddress HostPort Target
    
#    perform action for all applications for all XDAQ executives

    set i 0
    foreach ID $HostIDs {
        set HostAddress [lindex $HostAddresses $i]
        set HostPort [lindex $HostPorts $i]
  
        set TargetAddressList [lindex $TargetAddresses $i]
  
        SOAP::configure XDAQ_$action -proxy http://$HostAddress:$HostPort/

        foreach TargetAddr $TargetAddressList {
           set LastMethodName XDAQ_$action
	   set Target $TargetAddr
           set z [catch {XDAQ_$action -attr "targetAddr $TargetAddr"} m]
           if {$z != 0} then {XDAQ_Error "$m"} else {insert "$LastMethodName=> $m"}
        }
  
        incr i
    }
}

proc disableapplications {args} {
    global HostIDs HostAddresses HostPorts
    global DisableSequence
    global LastMethodName
    global HostAddress HostPort Target
    
#    perform action for all applications for all XDAQ executives

    foreach item $DisableSequence {

      set host [first $item]
      set TargetAddress [second $item]
  
      set i [lsearch $HostIDs $host]
  
      set HostAddress [lindex $HostAddresses $i]
      set HostPort [lindex $HostPorts $i]
  
      SOAP::configure XDAQ_Disable -proxy http://$HostAddress:$HostPort/
      set LastMethodName XDAQ_Disable
      set Target $TargetAddress
      set z [catch {XDAQ_Disable -attr "targetAddr $TargetAddress"} m]
      if {$z != 0} then {XDAQ_Error "$m"} else {insert "$LastMethodName=> $m"}
    }
}

proc enableapplications {args} {
    global HostIDs HostAddresses HostPorts
    global EnableSequence
    global LastMethodName
    global HostAddress HostPort Target
    
#    perform action for all applications for all XDAQ executives

    foreach item $EnableSequence {

      set host [first $item]
      set TargetAddress [second $item]
  
      set i [lsearch $HostIDs $host]
  
      set HostAddress [lindex $HostAddresses $i]
      set HostPort [lindex $HostPorts $i]
  
      SOAP::configure XDAQ_Enable -proxy http://$HostAddress:$HostPort/
      set LastMethodName XDAQ_Enable
      set Target $TargetAddress
      set z [catch {XDAQ_Enable -attr "targetAddr $TargetAddress"} m]
      if {$z != 0} then {XDAQ_Error "$m"} else {insert "$LastMethodName=> $m"}
    }
}

proc read_parameter {Host Addr Name} {
    global HostIDs HostAddresses HostPorts
    global HostID TargetAddress
    global LastMethodName
    global HostAddress HostPort Target
    
#    perform action for the specified application
    
    set i [lsearch $HostIDs $Host]
    
    set HostAddress [lindex $HostAddresses $i]
    set HostPort [lindex $HostPorts $i]
    
    SOAP::configure XDAQ_ReadParameter -proxy http://$HostAddress:$HostPort/
    SOAP::configure XDAQ_ReadParameter -params [list $Name string]
    set LastMethodName "XDAQ_ReadParameter $Name"
    set Target $Addr
    set z [catch {XDAQ_ReadParameter -attr "targetAddr $Addr" ""} m]
    if {$z != 0} then {XDAQ_Error "$m"} else {insert "$LastMethodName=> $m"}
    return $m
}

proc write_parameter {Host Addr Name vl} {
    global HostIDs HostAddresses HostPorts
    global HostID TargetAddress
    global LastMethodName
    global HostAddress HostPort Target
    
#    perform action for the specified application
    
    set i [lsearch $HostIDs $Host]
    
    set HostAddress [lindex $HostAddresses $i]
    set HostPort [lindex $HostPorts $i]
    
    SOAP::configure XDAQ_WriteParameter -proxy http://$HostAddress:$HostPort/
    SOAP::configure XDAQ_WriteParameter -params [list $Name string]
    set LastMethodName "XDAQ_WriteParameter $Name $vl"
    set Target $Addr
    set z [catch {XDAQ_WriteParameter -attr "targetAddr $Addr" $vl} m]
    if {$z != 0} then {XDAQ_Error "$m"} else {insert "$LastMethodName=> $m"}
    return $m
}

proc click_request {args} {
    global LastMethodName
    insert "[SOAP::dump -request [first $LastMethodName]]"
}

proc click_reply {args} {
    global LastMethodName
    insert "[SOAP::dump -reply [first $LastMethodName]]"
}

proc insert {s} {:modify LIST :strings "$s\n"}

proc host_action {v} {
   global HostID
    switch $v {
        0   {configureallhosts}
        1   {configurecurrenthost}
	2   {read_parameter $HostID 0 stateName}
        3   {do_currenthost Halt}
        4   {do_currenthost Reset}
        5   {do_currenthost Query}
        default {}
    }
}

proc appl_action {v} {
   global HostID TargetAddress 
    switch $v {
        0   {do_applications Configure}
        1   {do_currentapplication Configure}
        2   {read_parameter $HostID $TargetAddress stateName} 
        3   {click_displaycurrentapplication}
        4   {enableapplications}
        5   {disableapplications}
        6   {do_currentapplication Enable}
        7   {do_currentapplication Disable}
        8   {do_currentapplication Halt}
        9   {do_currentapplication Reset}
        10  {do_currentapplication Query}
        default {} 
    }
}

proc click arg {
   global HostID TargetAddress
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   switch $w  {
      HOSTS    {set HostID $i}
      APPLS    {set TargetAddress $i}
      HOSTACTIONS  {host_action $v}
      APPLACTIONS  {appl_action $v}
      default     {}
   }
   redisplay
}

proc start args {midas_default_frame}

proc start args {
   global started env
   upvar #0 browserdirectory directory

   if {$started == 0} then {
      set started 1  
      set directory [file join $env(MIDASBASE) Prisma XDAQ]
      cd $directory
      click_reload
      make-frame
   } else {
      open-frame
   }
   redisplay
}

